@@ -1,38 +1,30 @@
-import re
-import datetime
-import time
+from ._compat import PY2, text_type, long_type, JYTHON, IRONPYTHON
+
 import sys
+import binascii
+import datetime
+from decimal import Decimal
+import re
+import time
 
-from constants import FIELD_TYPE, FLAG
-from charset import charset_by_id
+from .constants import FIELD_TYPE, FLAG
+from .charset import charset_by_id, charset_to_encoding
 
-PYTHON3 = sys.version_info[0] > 2
-
-try:
-    set
-except NameError:
-    try:
-        from sets import BaseSet as set
-    except ImportError:
-        from sets import Set as set
 
 ESCAPE_REGEX = re.compile(r"[\0\n\r\032\'\"\\]")
 ESCAPE_MAP = {'\0': '\\0', '\n': '\\n', '\r': '\\r', '\032': '\\Z',
               '\'': '\\\'', '"': '\\"', '\\': '\\\\'}
 
+
 def escape_item(val, charset):
     if type(val) in [tuple, list, set]:
         return escape_sequence(val, charset)
     if type(val) is dict:
         return escape_dict(val, charset)
-    if PYTHON3 and hasattr(val, "decode") and not isinstance(val, unicode):
-        # deal with py3k bytes
-        val = val.decode(charset)
     encoder = encoders[type(val)]
     val = encoder(val)
-    if type(val) is str:
+    if type(val) in [str, int, text_type]:
         return val
-    val = val.encode(charset)
     return val
 
 def escape_dict(val, charset):
@@ -59,17 +51,25 @@ def escape_bool(value):
 def escape_object(value):
     return str(value)
 
-escape_int = escape_long = escape_object
+def escape_int(value):
+    return str(value)
+
 
 def escape_float(value):
     return ('%.15g' % value)
 
 def escape_string(value):
-    return ("'%s'" % ESCAPE_REGEX.sub(
-            lambda match: ESCAPE_MAP.get(match.group(0)), value))
+    return ("%s" % (ESCAPE_REGEX.sub(
+            lambda match: ESCAPE_MAP.get(match.group(0)), value),))
+
+def escape_str(value):
+    return "'%s'" % escape_string(value)
 
 def escape_unicode(value):
-    return escape_string(value)
+    return escape_str(value)
+
+def escape_bytes(value):
+    return "x'%s'" % binascii.hexlify(value).decode(sys.getdefaultencoding())
 
 def escape_None(value):
     return 'NULL'
@@ -78,7 +78,7 @@ def escape_timedelta(obj):
     seconds = int(obj.seconds) % 60
     minutes = int(obj.seconds // 60) % 60
     hours = int(obj.seconds // 3600) % 24 + int(obj.days) * 24
-    return escape_string('%02d:%02d:%02d' % (hours, minutes, seconds))
+    return escape_str('%02d:%02d:%02d' % (hours, minutes, seconds))
 
 def escape_time(obj):
     s = "%02d:%02d:%02d" % (int(obj.hour), int(obj.minute),
@@ -86,18 +86,18 @@ def escape_time(obj):
     if obj.microsecond:
         s += ".%f" % obj.microsecond
 
-    return escape_string(s)
+    return escape_str(s)
 
 def escape_datetime(obj):
-    return escape_string(obj.strftime("%Y-%m-%d %H:%M:%S"))
+    return escape_str(obj.strftime("%Y-%m-%d %H:%M:%S"))
 
 def escape_date(obj):
-    return escape_string(obj.strftime("%Y-%m-%d"))
+    return escape_str(obj.strftime("%Y-%m-%d"))
 
 def escape_struct_time(obj):
     return escape_datetime(datetime.datetime(*obj[:6]))
 
-def convert_datetime(connection, field, obj):
+def convert_datetime(obj):
     """Returns a DATETIME or TIMESTAMP column value as a datetime object:
 
       >>> datetime_or_None('2007-02-25 23:06:20')
@@ -113,22 +113,21 @@ def convert_datetime(connection, field, obj):
       True
 
     """
-    if not isinstance(obj, unicode):
-        obj = obj.decode(connection.charset)
     if ' ' in obj:
         sep = ' '
     elif 'T' in obj:
         sep = 'T'
     else:
-        return convert_date(connection, field, obj)
+        return convert_date(obj)
 
     try:
         ymd, hms = obj.split(sep, 1)
         return datetime.datetime(*[ int(x) for x in ymd.split('-')+hms.split(':') ])
     except ValueError:
-        return convert_date(connection, field, obj)
+        return convert_date(obj)
 
-def convert_timedelta(connection, field, obj):
+
+def convert_timedelta(obj):
     """Returns a TIME column as a timedelta object:
 
       >>> timedelta_or_None('25:06:17')
@@ -147,8 +146,6 @@ def convert_timedelta(connection, field, obj):
     """
     try:
         microseconds = 0
-        if not isinstance(obj, unicode):
-            obj = obj.decode(connection.charset)
         if "." in obj:
             (obj, tail) = obj.split('.')
             microseconds = int(tail)
@@ -163,7 +160,7 @@ def convert_timedelta(connection, field, obj):
     except ValueError:
         return None
 
-def convert_time(connection, field, obj):
+def convert_time(obj):
     """Returns a TIME column as a time object:
 
       >>> time_or_None('15:06:17')
@@ -196,7 +193,7 @@ def convert_time(connection, field, obj):
     except ValueError:
         return None
 
-def convert_date(connection, field, obj):
+def convert_date(obj):
     """Returns a DATE column as a date object:
 
       >>> date_or_None('2007-02-26')
@@ -211,13 +208,12 @@ def convert_date(connection, field, obj):
 
     """
     try:
-        if not isinstance(obj, unicode):
-            obj = obj.decode(connection.charset)
         return datetime.date(*[ int(x) for x in obj.split('-', 2) ])
     except ValueError:
         return None
 
-def convert_mysql_timestamp(connection, field, timestamp):
+
+def convert_mysql_timestamp(timestamp):
     """Convert a MySQL TIMESTAMP to a Timestamp object.
 
     MySQL >= 4.1 returns TIMESTAMP in the same format as DATETIME:
@@ -238,11 +234,8 @@ def convert_mysql_timestamp(connection, field, timestamp):
       True
 
     """
-    if not isinstance(timestamp, unicode):
-        timestamp = timestamp.decode(connection.charset)
-
     if timestamp[4] == '-':
-        return convert_datetime(connection, field, timestamp)
+        return convert_datetime(timestamp)
     timestamp += "0"*(14-len(timestamp)) # padding
     year, month, day, hour, minute, second = \
         int(timestamp[:4]), int(timestamp[4:6]), int(timestamp[6:8]), \
@@ -255,7 +248,7 @@ def convert_mysql_timestamp(connection, field, timestamp):
 def convert_set(s):
     return set(s.split(","))
 
-def convert_bit(connection, field, b):
+def convert_bit(b):
     #b = "\x00" * (8 - len(b)) + b # pad w/ zeroes
     #return struct.unpack(">Q", b)[0]
     #
@@ -265,34 +258,26 @@ def convert_bit(connection, field, b):
 
 def convert_characters(connection, field, data):
     field_charset = charset_by_id(field.charsetnr).name
+    encoding = charset_to_encoding(field_charset)
     if field.flags & FLAG.SET:
-        return convert_set(data.decode(field_charset))
+        return convert_set(data.decode(encoding))
     if field.flags & FLAG.BINARY:
         return data
 
     if connection.use_unicode:
-        data = data.decode(field_charset)
+        data = data.decode(encoding)
     elif connection.charset != field_charset:
-        data = data.decode(field_charset)
-        data = data.encode(connection.charset)
+        data = data.decode(encoding)
+        data = data.encode(connection.encoding)
     return data
 
-def convert_int(connection, field, data):
-    return int(data)
-
-def convert_long(connection, field, data):
-    return long(data)
-
-def convert_float(connection, field, data):
-    return float(data)
-
 encoders = {
         bool: escape_bool,
         int: escape_int,
-        long: escape_long,
+        long_type: escape_int,
         float: escape_float,
-        str: escape_string,
-        unicode: escape_unicode,
+        str: escape_str,
+        text_type: escape_unicode,
         tuple: escape_sequence,
         list:escape_sequence,
         set:escape_sequence,
@@ -303,51 +288,47 @@ encoders = {
         datetime.timedelta : escape_timedelta,
         datetime.time : escape_time,
         time.struct_time : escape_struct_time,
+        Decimal: text_type,
         }
 
+
+def through(x):
+    return x
+
+if not PY2 or JYTHON or IRONPYTHON:
+    encoders[bytes] = escape_bytes
+
 decoders = {
         FIELD_TYPE.BIT: convert_bit,
-        FIELD_TYPE.TINY: convert_int,
-        FIELD_TYPE.SHORT: convert_int,
-        FIELD_TYPE.LONG: convert_long,
-        FIELD_TYPE.FLOAT: convert_float,
-        FIELD_TYPE.DOUBLE: convert_float,
-        FIELD_TYPE.DECIMAL: convert_float,
-        FIELD_TYPE.NEWDECIMAL: convert_float,
-        FIELD_TYPE.LONGLONG: convert_long,
-        FIELD_TYPE.INT24: convert_int,
-        FIELD_TYPE.YEAR: convert_int,
+        FIELD_TYPE.TINY: int,
+        FIELD_TYPE.SHORT: int,
+        FIELD_TYPE.LONG: int,
+        FIELD_TYPE.FLOAT: float,
+        FIELD_TYPE.DOUBLE: float,
+        FIELD_TYPE.DECIMAL: float,
+        FIELD_TYPE.NEWDECIMAL: float,
+        FIELD_TYPE.LONGLONG: int,
+        FIELD_TYPE.INT24: int,
+        FIELD_TYPE.YEAR: int,
         FIELD_TYPE.TIMESTAMP: convert_mysql_timestamp,
         FIELD_TYPE.DATETIME: convert_datetime,
         FIELD_TYPE.TIME: convert_timedelta,
         FIELD_TYPE.DATE: convert_date,
         FIELD_TYPE.SET: convert_set,
-        FIELD_TYPE.BLOB: convert_characters,
-        FIELD_TYPE.TINY_BLOB: convert_characters,
-        FIELD_TYPE.MEDIUM_BLOB: convert_characters,
-        FIELD_TYPE.LONG_BLOB: convert_characters,
-        FIELD_TYPE.STRING: convert_characters,
-        FIELD_TYPE.VAR_STRING: convert_characters,
-        FIELD_TYPE.VARCHAR: convert_characters,
-        #FIELD_TYPE.BLOB: str,
-        #FIELD_TYPE.STRING: str,
-        #FIELD_TYPE.VAR_STRING: str,
-        #FIELD_TYPE.VARCHAR: str
+        FIELD_TYPE.BLOB: through,
+        FIELD_TYPE.TINY_BLOB: through,
+        FIELD_TYPE.MEDIUM_BLOB: through,
+        FIELD_TYPE.LONG_BLOB: through,
+        FIELD_TYPE.STRING: through,
+        FIELD_TYPE.VAR_STRING: through,
+        FIELD_TYPE.VARCHAR: through,
+        FIELD_TYPE.DECIMAL: Decimal,
+        FIELD_TYPE.NEWDECIMAL: Decimal,
         }
-conversions = decoders  # for MySQLdb compatibility
 
-try:
-    # python version > 2.3
-    from decimal import Decimal
-    def convert_decimal(connection, field, data):
-        data = data.decode(connection.charset)
-        return Decimal(data)
-    decoders[FIELD_TYPE.DECIMAL] = convert_decimal
-    decoders[FIELD_TYPE.NEWDECIMAL] = convert_decimal
 
-    def escape_decimal(obj):
-        return unicode(obj)
-    encoders[Decimal] = escape_decimal
+# for MySQLdb compatibility
+conversions = decoders
 
-except ImportError:
-    pass
+def Thing2Literal(obj):
+    return escape_str(str(obj))
